from fnvhash import fnv1a_32
from collections import Counter, OrderedDict
import json
import math
from pathlib import Path
from hfinger.configs import AEVAL, CONNVAL, CONTENC, CACHECONT, TE, ACCPTCHAR, METHODS
import sys


def read_config(filename):
    with (Path(__file__).parent / "configs" / filename).open() as fin:
        return json.load(fin, object_pairs_hook=OrderedDict)


HDRL = read_config("headerslow.json")
CONTENTTYPE = read_config("content-type.json")
ACCPT = read_config("accept.json")


def entropy(bstr):
    p, lns = Counter(bstr), float(len(bstr))
    return -sum(count / lns * math.log(count / lns, 2) for count in p.values())


def get_entropy(bstr):
    return str(round(entropy(bstr), 1))


def get_length(bstr):
    return str(round(math.log10(len(bstr)), 1))


def get_hdr_case(hdr):
    if "-" in hdr:
        t = hdr.split("-")
        for i in t:
            if i[0].islower():
                # Is any first letter in compound header name written with lower case?
                return False
        return True
    else:
        return hdr[0].isupper()


# Checking method and proto version
def get_method_version(pkt):
    r_ver = ""
    r_meth = ""
    # Checking if HTTP version is provided, if not assuming it is HTTP 0.9 per www.w3.org/Protocols/HTTP/Request.html
    if " HTTP/" not in pkt[0]:
        r_ver = "9"
        # take first seven characters of the first line of request to look for method (methods have up to 7 chars)
        t2 = pkt[0][:7].upper().strip(" ")
        # if method shorter than 7 chars we will have part of URL in t2
        # we should find space between method and URL and cut the string on it
        it = t2.find(" ")
        meth = t2[:it]
        if it < 0:
            # method has 7 chars, so no need to cut t2
            meth = t2
        if meth in METHODS:
            r_meth = meth[:2]
    else:
        t = pkt[0].split(" HTTP/")
        t1 = t[0].lstrip(" ")
        # check if method is present by taking first 7 characters and searching there for method
        # (methods have up to 7 chars)
        t2 = t1[:7].upper().strip(" ")
        # if method shorter than 7 chars we will have part of URL in t2
        # we should find space between method and URL and cut the string on it
        it = t2.find(" ")
        meth = t2[:it]
        if it < 0:
            # method has 7 chars, so no need to cut t2
            meth = t2
        if meth in METHODS:
            r_meth = meth[:2]
            if "1.1" in t[1]:
                r_ver = "1"
            else:
                r_ver = "0"
    return "|".join((r_meth, r_ver))


# Checking header order - assuming that header field contains ":"
def get_hdr_order(pkt):
    ret = []
    for i in pkt[1:]:
        t = i.split(":")[0]
        t_low = t.lower()
        k = format(fnv1a_32(t.encode()), "x")
        if t_low in HDRL:
            if get_hdr_case(t):
                k = HDRL[t_low]
            else:
                k = "!" + HDRL[t_low]
        ret.append(k)
    return ",".join(ret)


def get_ua_value(hdr):
    val = hdr.split(":")[1]
    if val[0] == " ":
        val = val[1:]
    name = HDRL["user-agent"]
    ret = name + ":" + format(fnv1a_32(val.encode()), "x")
    return ret


def get_hdr_value(hdr, hdrname, hdr_value_table):
    val = hdr.split(":")[1]
    if val[0] == " ":
        val = val[1:]
    hdr_coded = HDRL[hdrname] + ":"
    ret = []
    if "," in val:
        # simple splitting of compound values
        if ";q=" in val:
            # we do not tokenize compound values with quality parameters at this moment
            return hdr_coded + format(fnv1a_32(val.encode()), "x")
        t = []
        if ", " in val:
            t = val.split(", ")
        else:
            t = val.split(",")
        for j in t:
            if j == "":
                return hdr_coded + format(fnv1a_32(val.encode()), "x")
            if j not in hdr_value_table:
                print("Unknown header value - " + hdr, file=sys.stderr)
                return hdr_coded + format(fnv1a_32(val.encode()), "x")
            ret.append(hdr_value_table[j])
    else:
        k = ""
        if val in hdr_value_table:
            k = hdr_value_table[val]
        else:
            print("Unknown header value - " + hdr, file=sys.stderr)
            k = format(fnv1a_32(val.encode()), "x")
        ret.append(k)
    return hdr_coded + ",".join(ret)


def get_content_type(hdr):
    val = hdr.split(":")[1]
    if val[0] == " ":
        val = val[1:]
    hdr_coded = HDRL["content-type"] + ":"
    ret = []
    if "," in val:
        vals = []
        if ", " in val:
            vals = val.split(", ")
        else:
            vals = val.split(",")
        for itv in vals:
            if ";" in itv:
                if "boundary=" in itv:
                    bnd_ind = itv.index("boundary=")
                    bnd_offset = len("boundary=")
                    val_bnd = val[: bnd_ind + bnd_offset]
                    return hdr_coded + format(fnv1a_32(val_bnd.encode()), "x")
                else:
                    ret.append(format(fnv1a_32(itv.encode()), "x"))
            else:
                k = format(fnv1a_32(itv.encode()), "x")
                if itv not in CONTENTTYPE:
                    print("Unknown Content-Type value - " + hdr, file=sys.stderr)
                else:
                    k = CONTENTTYPE[itv]
                ret.append(k)
    else:
        if ";" in val:
            if "boundary=" not in val:
                return hdr_coded + format(fnv1a_32(val.encode()), "x")
            bnd_ind = val.index("boundary=")
            bnd_offset = len("boundary=")
            val_bnd = val[: bnd_ind + bnd_offset]
            return hdr_coded + format(fnv1a_32(val_bnd.encode()), "x")
        else:
            k = format(fnv1a_32(val.encode()), "x")
            if val not in CONTENTTYPE:
                print("Unknown Content-Type value - " + hdr, file=sys.stderr)
            else:
                k = CONTENTTYPE[val]
            ret.append(k)
    return hdr_coded + ",".join(ret)


def get_accept_language_value(hdr):
    val = hdr.split(":")[1]
    name = HDRL["accept-language"]
    ret = name + ":" + format(fnv1a_32(val.encode()), "x")
    return ret


def get_pop_hdr_val(pkt):
    r = []
    for i in pkt[1:]:
        t = i.split(":")[0].lower()
        if t == "connection":
            r.append(get_hdr_value(i, "connection", CONNVAL))
        elif t == "accept-encoding":
            r.append(get_hdr_value(i, "accept-encoding", AEVAL))
        elif t == "content-encoding":
            r.append(get_hdr_value(i, "content-encoding", CONTENC))
        elif t == "cache-control":
            r.append(get_hdr_value(i, "cache-control", CACHECONT))
        elif t == "te":
            r.append(get_hdr_value(i, "te", TE))
        elif t == "accept-charset":
            r.append(get_hdr_value(i, "accept-charset", ACCPTCHAR))
        elif t == "content-type":
            r.append(get_content_type(i))
        elif t == "accept":
            r.append(get_hdr_value(i, "accept", ACCPT))
        elif t == "accept-language":
            r.append(get_accept_language_value(i))
        elif t == "user-agent":
            r.append(get_ua_value(i))
    ret = "/".join(r)
    return ret



import subprocess
from distutils.version import LooseVersion


def get_tshark_version():
    version_raw = subprocess.check_output(["tshark", "-v"]).decode("ascii")
    for lin in version_raw.splitlines():
        if "TShark (Wireshark)" in lin:
            tshark_ver_full = lin.split("(")[1].split(")")[1].strip()
            return tshark_ver_full
        else:
            return "0.0.0"


def check_tshark_supports_json(tshark_ver):
    if LooseVersion(tshark_ver) >= LooseVersion("2.2.0"):
        return True
    else:
        return False


# Tshark before version 2.2.6 does not put "]" at the end of the json
def repair_json(tshark_ver, json_path):
    if LooseVersion(tshark_ver) < LooseVersion("2.2.6"):
        with open(json_path, "a") as repaired_json:
            repaired_json.write("\n ] \n")



import urllib.parse
import os
from math import log10
from pathlib import Path
from hfinger.configs import METHODS


extensions = set()
with (Path(__file__).parent / "configs/extensions.txt").open() as ext_in:
    for lin in ext_in:
        extensions.add(lin.strip("\n"))


def avg_length_dir(uri_t, numb):
    avg = 0
    for i in uri_t[1:]:
        avg = avg + len(i)
    if float(numb) > 0.0:
        return float(avg) / float(numb)
    else:
        return 0.0


def give_extension(uri_path):
    tmp_ext = os.path.splitext(uri_path)[1].strip(".")
    # naive approach to filter out strings with dot, which are not extensions
    # firstly check if tested string is between 1 and 4 characters
    if 1 <= len(tmp_ext) <= 4:
        if tmp_ext.isalnum() and tmp_ext in extensions:
            return tmp_ext
    return ""


def variables_features(qry):
    query_parsed = urllib.parse.parse_qs(qry)
    var_len = len(qry)
    var_number = len(query_parsed)
    val_len = 0
    for j in query_parsed.values():
        val_len = val_len + len(j[0])
    if var_number > 0:
        avg_len = log10(float(val_len) / float(var_number))
    else:
        avg_len = 0.0
    return "|".join(
        map(
            str,
            ("%.1f" % round(log10(var_len), 1), var_number, "%.1f" % round(avg_len, 1)),
        )
    )


def give_fing(uri):
    uri_parsed = urllib.parse.urlparse(uri)
    path_tokenized = uri_parsed.path.split("/")
    uri_len = len(uri)
    uri_len_log = log10(uri_len)
    var_feat = "||"
    if uri_len > 1:
        dir_number = (
            len(uri_parsed.path.split("/")) - 1
        )  # uri begins with "/", so split gives empty string before "/" as a value
        avg_dir_len = avg_length_dir(path_tokenized, dir_number)
        avg_dir_len_log = 0.0
        if avg_dir_len != 0.0:
            avg_dir_len_log = log10(avg_dir_len)
        if uri_parsed.query != "":
            var_feat = variables_features(uri_parsed.query)
        ext = give_extension(uri_parsed.path)
        return "|".join(
            map(
                str,
                (
                    "%.1f" % round(uri_len_log, 1),
                    dir_number,
                    "%.1f" % round(avg_dir_len_log, 1),
                    ext,
                    var_feat,
                ),
            )
        )
    else:
        return "|".join(
            map(str, ("%.1f" % round(uri_len_log, 1), "", "", "", "", "", ""))
        )


def uri_fingerprint(pkt):
    t = ""
    # Checking if request has protocol version declaration
    if " HTTP/" not in pkt[0]:
        t = pkt[0]
    else:
        t = pkt[0].split(" HTTP/")[0].lstrip(" ")
    # check if method is present in the first seven chars (methods have up to 7 chars)
    t1 = t[:7].upper().strip(" ")
    # find spaces in methods shorter than 7 chars - we will have part of URL in t1
    # we should find whitespace between method and URL and cut the string on it
    it = t1.find(" ")
    meth = t1[:it]
    # workaround for methods with 7 chars
    if it < 0:
        # method has 7 chars, so no need to cut t1
        meth = t1
    if meth in METHODS:
        # searching for URL, which should be separated from method with whitespace
        it1 = t.find(" ")
        # cutting out URL
        uri = t[it1 + 1 :]
        return give_fing(uri)
    else:
        return "|||||||"



import sys
import shutil
import tempfile
import os
import argparse
import magic
import subprocess
import json

from hfinger import hreader, tshark_wrappers
from .hfinger_exceptions import (
    NotAPcap,
    BadReportmodeVariable,
    PythonTooOld,
    TsharkNotFound,
    TsharkTooOld,
)


def ensure_environment():
    if sys.version_info < (3, 3):
        raise PythonTooOld
    locate = shutil.which("tshark")
    if locate is None:
        raise TsharkNotFound
    tshark_exec = locate
    tshark_ver = tshark_wrappers.get_tshark_version()
    if not tshark_wrappers.check_tshark_supports_json(tshark_ver):
        raise TsharkTooOld
    return tshark_exec, tshark_ver


def is_pcap_file(file_path):
    f_type = magic.from_file(file_path, mime=True)
    if f_type != "application/vnd.tcpdump.pcap":
        raise NotAPcap


def write_results_to_file(filename, outpath, results_to_write):
    n1 = os.path.basename(filename)
    path_to_file = os.path.join(outpath, n1 + ".json")
    with open(path_to_file, "w") as f_out:
        json.dump(results_to_write, f_out)


def run_tshark(pcap_path, rep_mode, texec, tver):
    with tempfile.TemporaryDirectory() as tmpdirname:
        fname = os.path.join(tmpdirname, "json.out")
        with open(fname, "w") as json_out_file:
            subprocess.call(
                [
                    texec,
                    "-T",
                    "json",
                    "-x",
                    "-Yhttp.request and tcp and not icmp",
                    "-r",
                    pcap_path,
                ],
                stdout=json_out_file,
            )
        tshark_wrappers.repair_json(tver, fname)
        return hreader.reader_wrapper(fname, rep_mode)


def commandline_run():
    my_parser = argparse.ArgumentParser(
        description="Hfinger - fingerprinting malware HTTP requests stored in pcap files",
        allow_abbrev=False,
        prog="hfinger",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    my_group = my_parser.add_mutually_exclusive_group(required=True)
    my_group.add_argument(
        "-f", "--file", action="store", type=str, help="Read a single pcap file"
    )
    my_group.add_argument(
        "-d",
        "--directory",
        metavar="DIR",
        action="store",
        type=str,
        help="Read pcap files from the directory DIR",
    )
    my_parser.add_argument(
        "-o",
        "--output-path",
        metavar="output_path",
        type=str,
        action="store",
        help="Path to the output directory",
    )
    my_parser.add_argument(
        "-m",
        "--mode",
        type=int,
        default=2,
        choices=[0, 1, 2, 3, 4],
        help="Fingerprint report mode. "
        "\n0 - similar number of collisions and fingerprints as mode 2, but using fewer features, "
        "\n1 - representation of all designed features, but a little more collisions than modes 0, 2, and 4, "
        "\n2 - optimal (the default mode), "
        "\n3 - the lowest number of generated fingerprints, but the highest number of collisions, "
        "\n4 - the highest fingerprint entropy, but slightly more fingerprints than modes 0-2",
    )
    args = my_parser.parse_args()
    tshark_exec = ""
    tshark_ver = ""
    try:
        tshark_exec, tshark_ver = ensure_environment()
    except (PythonTooOld, TsharkNotFound, TsharkTooOld) as err:
        print(err)
        sys.exit(1)
    if args.file:
        try:
            is_pcap_file(args.file)
        except (FileNotFoundError, IsADirectoryError, PermissionError) as err:
            print("Problem with file access. " + str(err))
            sys.exit(1)
        except NotAPcap:
            print("The provided file is not a valid pcap file.")
            sys.exit(1)
        else:
            results = run_tshark(args.file, args.mode, tshark_exec, tshark_ver)
            if args.output_path is not None:
                write_results_to_file(args.file, args.output_path, results)
            else:
                print(json.dumps(results))
    else:
        no_pcaps_found_flag = True
        try:
            filelist = os.listdir(args.directory)
        except NotADirectoryError as err:
            print("The entered path is not a directory. " + str(err))
            sys.exit(1)
        for x in os.listdir(args.directory):
            cur_file = os.path.join(args.directory, x)
            try:
                is_pcap_file(cur_file)
            except IsADirectoryError:
                continue
            except (FileNotFoundError, PermissionError) as err:
                print("Problem with file access. " + str(err))
                sys.exit(1)
            except NotAPcap:
                continue
            else:
                no_pcaps_found_flag = False
                results = run_tshark(cur_file, args.mode, tshark_exec, tshark_ver)
                if args.output_path is not None:
                    write_results_to_file(cur_file, args.output_path, results)
                else:
                    print(json.dumps(results))
        if no_pcaps_found_flag:
            print("No valid pcap files found in the directory")


def hfinger_analyze(pcap, reportmode=2):
    """
    Returns the results of fingerprinting for single pcap file.
    Intended to be called from python scripts as the main function of the tool.

            Parameters:
                    pcap (str): Path to pcap file.
                    reportmode (int): Reporting mode of Hfinger in range 0-4. Default value is '2'.

            Returns:
                    results (list): Python list of dicts with fingerprinting results.

            Raises:
                    HfingerReportmodeVariableException: The value of the reportmode is not an int or not in range 0-4

    """

    tshark_exec, tshark_ver = ensure_environment()
    is_pcap_file(pcap)
    if not isinstance(reportmode, int):
        raise BadReportmodeVariable("Wrong type, should be 'int'.")
    if reportmode not in range(0, 5):
        raise BadReportmodeVariable("Wrong value, should be in range 0-4.")
    results = run_tshark(pcap, reportmode, tshark_exec, tshark_ver)
    return results



import json
import sys
from hfinger.configs import FEATURESET
from hfinger import hfinger_core, uri_reader


def compute_finger_raw(tcp_raw, iscrlf):
    pkt = []
    if iscrlf:
        pkt = bytes.fromhex(tcp_raw).split(b"\r\n\r\n")[0].decode("ascii").split("\r\n")
    else:
        pkt = bytes.fromhex(tcp_raw).split(b"\n\n")[0].decode("ascii").split("\n")
    uri_fing = uri_reader.uri_fingerprint(pkt)
    method = hfinger_core.get_method_version(pkt)
    hdr = hfinger_core.get_hdr_order(pkt)
    val2 = hfinger_core.get_pop_hdr_val(pkt)
    return f"{uri_fing}|{method}|{hdr}|{val2}"


# format the fingerprint according to the reporting mode
def format_output(fing_r, fing_p, repmode):
    mask = FEATURESET[repmode]
    ret = []
    if fing_r == "NULL":
        return "NULL"
    if fing_p == "":
        fing_p = "||"
    fing_tmp = fing_r + "|" + fing_p
    fingtmps = fing_tmp.split("|")
    for i in mask:
        elt = fingtmps[int(i)]
        if mask[i] == "i" and elt != "":
            elt = str(int(round(float(elt), 0)))
        ret.append(elt)
    return "|".join(ret)


def analyze_request_give_fingerprint(tmp, crlftagpresent):
    payload_raw = ""
    finger_pay = ""
    delim = "0d0a0d0a"
    tag_len = 8
    if not crlftagpresent:
        delim = "0a0a"
        tag_len = 4
    finger_raw = compute_finger_raw(tmp, crlftagpresent)
    ind1 = tmp.find(delim)
    if len(tmp[ind1 + tag_len :]) > 0:
        payload_raw = tmp[ind1 + tag_len :]
    if payload_raw != "":
        try:
            payload_ascii = bytes.fromhex(payload_raw).decode("ascii")
        except UnicodeDecodeError:
            finger_pay += "N|"
        except TypeError:
            print(
                "Problem with string length when parsing payload",
                file=sys.stderr,
            )
        else:
            finger_pay += "A|"
        payload_bytes = bytes.fromhex(payload_raw)
        finger_pay += hfinger_core.get_entropy(payload_bytes)
        finger_pay += "|"
        finger_pay += hfinger_core.get_length(payload_bytes)
    return finger_raw, finger_pay


def give_fingerprint(treq):
    finger_raw = ""
    finger_pay = ""
    if "0d0a0d0a" in treq:
        finger_raw, finger_pay = analyze_request_give_fingerprint(treq, True)
    elif "0a0a" in treq:
        print(
            "No CRLFCRLF in request switching to LFLF",
            file=sys.stderr,
        )
        finger_raw, finger_pay = analyze_request_give_fingerprint(treq, False)
    else:
        print(
            "No CRLFCRLF or LFLF in request",
            file=sys.stderr,
        )
        finger_raw = "NULL"
    return finger_raw, finger_pay


def reader(data, report_mode, tsharkold):
    retlist = []
    for p in data:
        finger_raw = ""
        payload_raw = ""
        finger_pay = ""
        json_dict = {}
        # Is http_raw layer present in the json dump?
        if "http_raw" in p["_source"]["layers"]:
            # Is tcp.segments_raw layer present in the json dump?
            if "tcp.segments_raw" in p["_source"]["layers"]:
                tmp = ""
                if tsharkold:
                    tmp = p["_source"]["layers"]["tcp.segments_raw"]
                else:
                    tmp = p["_source"]["layers"]["tcp.segments_raw"][0]
                finger_raw, finger_pay = give_fingerprint(tmp)
            else:
                # No tcp.segments_raw layer, trying request extraction from frame_raw
                tmp = ""
                if tsharkold:
                    tcpr = p["_source"]["layers"]["tcp_raw"]
                    ipr = p["_source"]["layers"]["ip_raw"]
                    ethr = p["_source"]["layers"]["eth_raw"]
                    framer = p["_source"]["layers"]["frame_raw"]
                    delimr = ethr + ipr + tcpr
                    tmp = framer.split(delimr)[1]
                else:
                    tcpr = p["_source"]["layers"]["tcp_raw"][0]
                    ipr = p["_source"]["layers"]["ip_raw"][0]
                    ethr = p["_source"]["layers"]["eth_raw"][0]
                    framer = p["_source"]["layers"]["frame_raw"][0]
                    delimr = ethr + ipr + tcpr
                    tmp = framer.split(delimr)[1]
                finger_raw, finger_pay = give_fingerprint(tmp)
                if "0d0a0d0a" in tmp:
                    finger_raw, finger_pay = analyze_request_give_fingerprint(tmp, True)
                elif "0a0a" in tmp:
                    print(
                        "No CRLFCRLF in request when parsing using frame_raw - switching to LFLF",
                        file=sys.stderr,
                    )
                    finger_raw, finger_pay = analyze_request_give_fingerprint(
                        tmp, False
                    )
                else:
                    # Unusual situation, extracting request from http_raw layer - not always dependable layer
                    print(
                        "Some other problem when parsing new line tags - going back to http_raw",
                        file=sys.stderr,
                    )
                    tmp = p["_source"]["layers"]["http_raw"]
                    finger_raw, finger_pay = give_fingerprint(tmp)
        json_dict["epoch_time"] = p["_source"]["layers"]["frame"]["frame.time_epoch"]
        json_dict["ip_src"] = p["_source"]["layers"]["ip"]["ip.src"]
        json_dict["ip_dst"] = p["_source"]["layers"]["ip"]["ip.dst"]
        json_dict["port_src"] = p["_source"]["layers"]["tcp"]["tcp.srcport"]
        json_dict["port_dst"] = p["_source"]["layers"]["tcp"]["tcp.dstport"]
        json_dict["fingerprint"] = format_output(finger_raw, finger_pay, report_mode)
        retlist.append(json_dict)
    return retlist


# checking format of json to choose proper version of parsing
def reader_wrapper(fjson, rmode):
    with open(fjson, "r") as fin:
        data = json.load(fin)
        if not data:
            return []
        if len(data[0]["_source"]["layers"]["frame_raw"]) > 1:
            return reader(data, rmode, False)
        else:
            return reader(data, rmode, True)



from setuptools import setup

LONG_DESCRIPTION = """
# Hfinger - fingerprinting malware HTTP requests

Tool for fingerprinting malware HTTP requests. Based on Tshark and written in Python3. Working prototype stage :-)

It's main objective is to provide a representation of malware requests in a shorter form than printing whole request, 
but still human interpretable. This representation should be unique between malware families, 
what means that any fingerprint should be seen only for one particular family.

Project's website: [https://github.com/CERT-Polska/hfinger](https://github.com/CERT-Polska/hfinger).
"""

setup(
    name="hfinger",
    version="0.2.0",
    description="Hfinger - fingerprinting malware HTTP requests stored in pcap files",
    author="Piotr Bialczak",
    author_email="piotrb@cert.pl",
    packages=["hfinger"],
    url="https://github.com/CERT-Polska/hfinger",
    include_package_data=True,
    install_requires=open("requirements.txt").read().splitlines(),
    python_requires=">=3.3",
    entry_points={
        "console_scripts": [
            "hfinger = hfinger.analysis:commandline_run",
        ]
    },
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3",
    ],
)



class BadReportmodeVariable(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
        else:
            self.message = None

    def __str__(self):
        if self.message:
            return self.message
        else:
            return "Problem with 'reportmode' variable value."


class NotAPcap(Exception):
    def __str__(self):
        return "The provided file is not a valid pcap file."


class PythonTooOld(Exception):
    def __str__(self):
        return "Python version is too old. Must be run with at least Python 3.3"


class TsharkNotFound(Exception):
    def __str__(self):
        return "No Tshark instance found. Is it installed?"


class TsharkTooOld(Exception):
    def __str__(self):
        return "Tshark version is too old. At least Tshark 2.2.0 version required."
